#!/bin/sh
# Bryant Hansen

ME="$(basename "$0")"
USAGE="$ME"

SYNC_SRC_DIR="/tmp/logs"
CONF_FILE="/etc/logfile_uploader.conf"

DebugMessage() {
    local LOG_RECORD_HEADER="`date "+%Y%m%d_%H%M%S"` (${ME}):"
    echo "${LOG_RECORD_HEADER} $*" >&2
}

# Determine if a log file is rotated via the logrotate utility
#   check if the filename ends with a . followed by 1 or more numbers
is_logrotated() {
    expr match "$1" ".*\.[0-9][0-9]*$" > /dev/null && return 0 || return 1
}


if [[ ! -f "$CONF_FILE" ]] ; then
	DebugMessage "ERROR: configuration file $CONF_FILE not found."
	DebugMessage "Exiting abnormally."
	exit 2
fi
. "$CONF_FILE"

if [[ ! "$SYNC_SRC_DIR" ]] ; then
	DebugMessage "ERROR: SYNC_SRC_DIR not specified"
	DebugMessage "Exiting abnormally."
	exit 3
fi	

if [[ -d "$SYNC_SRC_DIR" ]] ; then
	DebugMessage "SYNC_SRC_DIR $SYNC_SRC_DIR already exists.  Exiting without doing anything."
	exit 0
fi
echo "mkdir -p \"${SYNC_SRC_DIR}\""

# start by verifying that there's a link to apartments.xml
ent=/usr/share/dss/data/apartment.xml
if [[ -f "$ent" ]] ; then
	dir="$(dirname "$ent")"
	destdir="${SYNC_SRC_DIR}/$(dirname "$ent")"
    if [[ ! -d "$destdir" ]] ; then
            echo "mkdir -p \"${destdir}\""
    fi
    echo "ln -s \"${ent}\" \"${SYNC_SRC_DIR}/${ent}\""
else
	DebugMessage "$ent not found.  The system does not appear to be properly configured."
fi

find /var/log |
while read ent ; do
	if [[ -d "$ent" ]] ; then
		if [[ ! -d "$SYNC_SRC_DIR"/"$ent" ]] ; then
			d="$SYNC_SRC_DIR"/"$ent"
			[[ ! -d "$d" ]] && echo "mkdir -p \"${d}\""
		fi
	fi
	if [[ -f "$ent" ]] ; then
		dir="$(dirname "$ent")"
		d="$SYNC_SRC_DIR"/"$dir"
		[[ ! -d "$d" ]] && echo "mkdir -p \"${d}\""
		# detect if this is a logrotated file
		if is_logrotated "$ent" ; then
            #echo "$ent is a logrotate file" >&2
            if which stat >/dev/null 2>&1 ; then
                dt=$(stat -c "%X" "$ent")
                if [[ ! "$dt" ]] ; then
                    # echo "failed to get the file date for some unknown reason." >&2
                    # echo "using the first 8 digits of the md5 instead" >&2
                    md5="$(md5sum "$ent" | cut -f 1 -d ' ')"
                    md5=${md5:0:8}
                    dt=$md5
                fi
                dest="${SYNC_SRC_DIR}/${ent%\.[0-9]*}.${dt}"
            else
                md5="$(md5sum "$ent" | cut -f 1 -d ' ')"
                dest="${SYNC_SRC_DIR}/${ent%\.[0-9]*}.${md5}"
            fi
			echo "ln -s \"${ent}\" \"${dest}\""
		else
			#echo "$ent is not a logrotate file" >&2
			dest="${SYNC_SRC_DIR}/${ent}"
			echo "ln -s \"${ent}\" \"${dest}\""
		fi
	fi
done

echo "mkdir \"${SYNC_SRC_DIR}\"/smemcap"
echo "smemcap | gzip > \"${SYNC_SRC_DIR}\"/smemcap/$(date "+%Y%m%d_%H%M%S").tar.gz"

echo "mkdir \"${SYNC_SRC_DIR}\"/df"
echo "df 2>&1 > \"${SYNC_SRC_DIR}\"/df/$(date "+%Y%m%d_%H%M%S").log"

echo "mkdir \"${SYNC_SRC_DIR}\"/ps"
if ps auwx > /dev/null 2>&1 ; then
    # full (procps) version of ps
    echo "ps auwx 2>&1 > \"${SYNC_SRC_DIR}\"/ps/$(date "+%Y%m%d_%H%M%S").log"
else
    # busybox version of ps
    echo "ps w 2>&1 > \"${SYNC_SRC_DIR}\"/ps/$(date "+%Y%m%d_%H%M%S").log"
fi

# tighten up the security
echo "chmod -R go-rwx \"${SYNC_SRC_DIR}\""
